@@ -42,6 +42,7 @@
  */
 
 #include <drm/drmP.h>
+#include <drm/drm_mm_hsw.h>
 #include <linux/slab.h>
 #include <linux/seq_file.h>
 #include <linux/export.h>
@@ -73,8 +74,8 @@ static struct drm_mm_node_hsw *drm_mm_kmalloc_hsw(struct drm_mm_hsw *mm, int ato
 	return child;
 }
 
-/* drm_mm_pre_get_hsw() - pre allocate drm_mm_node structure
- * drm_mm:	memory manager struct we are pre-allocating for
+/* drm_mm_pre_get_hsw() - pre allocate drm_mm_node_hsw structure
+ * drm_mm_hsw:	memory manager struct we are pre-allocating for
  *
  * Returns 0 on success or -ENOMEM if allocation fails.
  */
@@ -183,19 +184,27 @@ EXPORT_SYMBOL(drm_mm_get_block_generic_hsw);
  * -ENOSPC if no suitable free area is available. The preallocated memory node
  * must be cleared.
  */
-int drm_mm_insert_node_hsw(struct drm_mm_hsw *mm, struct drm_mm_node_hsw *node,
-		       unsigned long size, unsigned alignment)
+int drm_mm_insert_node_generic_hsw(struct drm_mm_hsw *mm, struct drm_mm_node_hsw *node,
+			       unsigned long size, unsigned alignment,
+			       unsigned long color)
 {
 	struct drm_mm_node_hsw *hole_node;
 
-	hole_node = drm_mm_search_free_generic_hsw(mm, size, alignment, 0, false);
+	hole_node = drm_mm_search_free_generic_hsw(mm, size, alignment,
+					       color, 0);
 	if (!hole_node)
 		return -ENOSPC;
 
-	drm_mm_insert_helper_hsw(hole_node, node, size, alignment, 0);
-
+	drm_mm_insert_helper_hsw(hole_node, node, size, alignment, color);
 	return 0;
 }
+EXPORT_SYMBOL(drm_mm_insert_node_generic_hsw);
+
+int drm_mm_insert_node_hsw(struct drm_mm_hsw *mm, struct drm_mm_node_hsw *node,
+		       unsigned long size, unsigned alignment)
+{
+	return drm_mm_insert_node_generic_hsw(mm, node, size, alignment, 0);
+}
 EXPORT_SYMBOL(drm_mm_insert_node_hsw);
 
 static void drm_mm_insert_helper_range_hsw(struct drm_mm_node_hsw *hole_node,
@@ -212,11 +221,13 @@ static void drm_mm_insert_helper_range_hsw(struct drm_mm_node_hsw *hole_node,
 
 	BUG_ON(!hole_node->hole_follows || node->allocated);
 
-	if (mm->color_adjust)
-		mm->color_adjust(hole_node, color, &adj_start, &adj_end);
-
 	if (adj_start < start)
 		adj_start = start;
+	if (adj_end > end)
+		adj_end = end;
+
+	if (mm->color_adjust)
+		mm->color_adjust(hole_node, color, &adj_start, &adj_end);
 
 	if (alignment) {
 		unsigned tmp = adj_start % alignment;
@@ -274,21 +285,30 @@ EXPORT_SYMBOL(drm_mm_get_block_range_generic_hsw);
  * -ENOSPC if no suitable free area is available. This is for range
  * restricted allocations. The preallocated memory node must be cleared.
  */
+int drm_mm_insert_node_in_range_generic_hsw(struct drm_mm_hsw *mm, struct drm_mm_node_hsw *node,
+					unsigned long size, unsigned alignment, unsigned long color,
+					unsigned long start, unsigned long end)
+{
+	struct drm_mm_node_hsw *hole_node;
+
+	hole_node = drm_mm_search_free_in_range_generic_hsw(mm,
+							size, alignment, color,
+							start, end, 0);
+	if (!hole_node)
+		return -ENOSPC;
+
+	drm_mm_insert_helper_range_hsw(hole_node, node,
+				   size, alignment, color,
+				   start, end);
+	return 0;
+}
+EXPORT_SYMBOL(drm_mm_insert_node_in_range_generic_hsw);
+
 int drm_mm_insert_node_in_range_hsw(struct drm_mm_hsw *mm, struct drm_mm_node_hsw *node,
 				unsigned long size, unsigned alignment,
 				unsigned long start, unsigned long end)
 {
-	struct drm_mm_node_hsw *hole_node;
-
-	hole_node = drm_mm_search_free_in_range_generic_hsw(mm, size, alignment, 0,
-						start, end, false);
-	if (!hole_node)
-		return -ENOSPC;
-
-	drm_mm_insert_helper_range_hsw(hole_node, node, size, alignment, 0,
-				   start, end);
-
-	return 0;
+	return drm_mm_insert_node_in_range_generic_hsw(mm, node, size, alignment, 0, start, end);
 }
 EXPORT_SYMBOL(drm_mm_insert_node_in_range_hsw);
 
@@ -327,8 +347,8 @@ EXPORT_SYMBOL(drm_mm_remove_node_hsw);
 
 /*
  * Remove a memory node from the allocator and free the allocated struct
- * drm_mm_node. Only to be used on a struct drm_mm_node_hsw obtained by one of the
- * drm_mm_get_block functions.
+ * drm_mm_node_hsw. Only to be used on a struct drm_mm_node_hsw obtained by one of the
+ * drm_mm_get_block_hsw functions.
  */
 void drm_mm_put_block_hsw(struct drm_mm_node_hsw *node)
 {
@@ -488,7 +508,7 @@ void drm_mm_init_scan_hsw(struct drm_mm_hsw *mm,
 	mm->scan_size = size;
 	mm->scanned_blocks = 0;
 	mm->scan_hit_start = 0;
-	mm->scan_hit_size = 0;
+	mm->scan_hit_end = 0;
 	mm->scan_check_range = 0;
 	mm->prev_scanned_node = NULL;
 }
@@ -515,7 +535,7 @@ void drm_mm_init_scan_with_range_hsw(struct drm_mm_hsw *mm,
 	mm->scan_size = size;
 	mm->scanned_blocks = 0;
 	mm->scan_hit_start = 0;
-	mm->scan_hit_size = 0;
+	mm->scan_hit_end = 0;
 	mm->scan_start = start;
 	mm->scan_end = end;
 	mm->scan_check_range = 1;
@@ -534,8 +554,7 @@ int drm_mm_scan_add_block_hsw(struct drm_mm_node_hsw *node)
 	struct drm_mm_hsw *mm = node->mm;
 	struct drm_mm_node_hsw *prev_node;
 	unsigned long hole_start, hole_end;
-	unsigned long adj_start;
-	unsigned long adj_end;
+	unsigned long adj_start, adj_end;
 
 	mm->scanned_blocks++;
 
@@ -552,14 +571,8 @@ int drm_mm_scan_add_block_hsw(struct drm_mm_node_hsw *node)
 	node->node_list.next = &mm->prev_scanned_node->node_list;
 	mm->prev_scanned_node = node;
 
-	hole_start = drm_mm_hole_node_start_hsw(prev_node);
-	hole_end = drm_mm_hole_node_end_hsw(prev_node);
-
-	adj_start = hole_start;
-	adj_end = hole_end;
-
-	if (mm->color_adjust)
-		mm->color_adjust(prev_node, mm->scan_color, &adj_start, &adj_end);
+	adj_start = hole_start = drm_mm_hole_node_start_hsw(prev_node);
+	adj_end = hole_end = drm_mm_hole_node_end_hsw(prev_node);
 
 	if (mm->scan_check_range) {
 		if (adj_start < mm->scan_start)
@@ -568,11 +581,14 @@ int drm_mm_scan_add_block_hsw(struct drm_mm_node_hsw *node)
 			adj_end = mm->scan_end;
 	}
 
+	if (mm->color_adjust)
+		mm->color_adjust(prev_node, mm->scan_color,
+				 &adj_start, &adj_end);
+
 	if (check_free_hole_hsw(adj_start, adj_end,
 			    mm->scan_size, mm->scan_alignment)) {
 		mm->scan_hit_start = hole_start;
-		mm->scan_hit_size = hole_end;
-
+		mm->scan_hit_end = hole_end;
 		return 1;
 	}
 
@@ -588,7 +604,7 @@ EXPORT_SYMBOL(drm_mm_scan_add_block_hsw);
  * corrupted.
  *
  * When the scan list is empty, the selected memory nodes can be freed. An
- * immediately following drm_mm_search_free with best_match = 0 will then return
+ * immediately following drm_mm_search_free_hsw with best_match = 0 will then return
  * the just freed block (because its at the top of the free_stack list).
  *
  * Returns one if this block should be evicted, zero otherwise. Will always
@@ -608,19 +624,10 @@ int drm_mm_scan_remove_block_hsw(struct drm_mm_node_hsw *node)
 			       node_list);
 
 	prev_node->hole_follows = node->scanned_preceeds_hole;
-	INIT_LIST_HEAD(&node->node_list);
 	list_add(&node->node_list, &prev_node->node_list);
 
-	/* Only need to check for containement because start&size for the
-	 * complete resulting free block (not just the desired part) is
-	 * stored. */
-	if (node->start >= mm->scan_hit_start &&
-	    node->start + node->size
-	    		<= mm->scan_hit_start + mm->scan_hit_size) {
-		return 1;
-	}
-
-	return 0;
+	 return (drm_mm_hole_node_end_hsw(node) > mm->scan_hit_start &&
+		 node->start < mm->scan_hit_end);
 }
 EXPORT_SYMBOL(drm_mm_scan_remove_block_hsw);
 
