@@ -330,6 +330,7 @@ enum intel_pch {
 #define QUIRK_LVDS_SSC_DISABLE (1<<1)
 #define QUIRK_INVERT_BRIGHTNESS (1<<2)
 #define QUIRK_RC6_DISABLE (1<<3)
+#define QUIRK_NO_PCH_PWM_ENABLE (1<<4)
 
 struct intel_fbdev;
 struct intel_fbc_work;
@@ -651,6 +652,7 @@ typedef struct drm_i915_private {
 		unsigned long gtt_start;
 		unsigned long gtt_mappable_end;
 		unsigned long gtt_end;
+		unsigned long stolen_base; /* limited to low memory (32-bit) */
 
 		struct io_mapping *gtt_mapping;
 		phys_addr_t gtt_base_addr;
